% function [VV] = draw_PWA_1D(alpha,beta,Jlow,Jup,V)
% Jup   - upper boundaries for optimal objective function 
% Jlow  - lower boundaries for optimal objective function 
% alpha - directive (f(x) = alpha*x + beta)
% beta  - shift (f(x) = alpha*x + beta)
% V     - vertices of each old region
% VV    - vertices of each new region (from new alpha and beta)
function [VV] = draw_PWA_1D(alpha,beta,Jlow,Jup,V)
nx = length(Jlow.C{1});
if nx > 2, error(sprintf('Cant plot %dD !',nx+1)),end

% find D - range of x axis (range of obj. function)
for k = 1:nx
    dd = [];
    for i = 1:length(Jlow.R)
        temp =  extreme(Jlow.R(i));
        dd = [dd temp(:,k)'];
    end
    % D = [min(min(dd)) max(max(dd))];
    D(k,:) = [min(dd) max(dd)];
end

hold on
if nx  == 1
    x = min(D):1e-5:max(D);
    y = zeros(length(x),1);

% draw: Jup & Jlow boundaries
    for i = 1:length(V)
        xx = V{i};
        yy = [xx(1)*Jup.C{i} + Jup.D{i}; xx(2)*Jup.C{i} + Jup.D{i}];
        zz = [xx(1)*Jlow.C{i} + Jlow.D{i}; xx(2)*Jlow.C{i} + Jlow.D{i}];
        
        plot(xx,yy,'--k','LineWidth',3)
        plot(xx,zz,'--k','LineWidth',3)
    end
    
% draw: approximated PWA(alpha beta)
    % searching for new regions (P1 = brk(1:2), P2 = brk(2:3),...)
    brk = [];
    positon = zeros(length(x),1);
    for i = 1:length(x)
        fx = zeros(length(alpha),1);
        for j = 1:length(alpha)
            fx(j) = alpha(j)*x(i) + beta(j);
        end
        y(i) = max(fx);
        temp = find(fx == max(fx)); positon(i) = temp(1);
        if i == 1 || i == length(x)
            brk = [brk i];
        elseif positon(i) ~= positon(i-1)
            brk = [brk i];
        end
    end

    for i = 1:length(brk)-1
        VV{i} = [x(brk(i));x(brk(i+1))];
    end
    
    for i = 1:length(brk)-1
        plot([x(brk(i)), x(brk(i+1))],[y(brk(i)),...
            y(brk(i+1))],'--r','LineWidth',3)
    end
    
    xlabel('x')
    ylabel('f(x)')
    title(sprintf('PWA function over %d regions',length(VV)))
end

