function [PJ, PJN, J, Jmax] = get_J_epigraph(R, B, Clow, Dlow, Cup, Dup)

nx = dimension(R(1));

% obtain maximal value of Jup(x) over domain "B"
V = extreme(B);
Jmax = 0;
for i = 1:length(Cup)
    for j = 1:size(V, 1)
        Jmax = max(Jmax, Cup{i}'*V(j, :)' + Dup{i});
    end
end

fprintf('Computing epigraph of Jlow(x)...\n');
% epigraph of the value function, i.e.
%  { [x; e] | e >= Jlow(x) }
x = sdpvar(nx, 1);
e = sdpvar(1, 1);
yPJ = [ ismember(x, B); e <= Jmax ];
for k = 1:length(Clow)
    yPJ = yPJ + [ e >= Clow{k}'*x + Dlow{k} ];
end
PJ = union(polytope(yPJ));

fprintf('Computing epigraph of Jup(x)...\n');
% epigraph of the shifted value function, i.e. 
%  { [x; e] | e >= J(x) + ||x|| }
yPJN = [ ismember(x, B); e <= Jmax ];
for k = 1:length(Cup)
    yPJN = yPJN + [ e >= Cup{k}'*x + Dup{k} ];
end
PJN = union(polytope(yPJN));

fprintf('Computing polytopes of Jup(x)-Jlow(x)...\n');
% polytopic representation of J(x)+||x|| - J(x)
J = polytope;
V = pelemfun(@polytope, R);
nR = length(R);
x = sdpvar(nx, 1);
e = sdpvar(1, 1);
for i = 1:nR
    if i==1 || i == nR || mod(i, 10)==0
        fprintf('%d / %d\n', i, nR);
    end
    Jlow = Clow{i}'*x+Dlow{i};
    Jup = Cup{i}'*x+Dup{i};
    yJ = [ ismember(x, R(i)); Jlow <= e <= Jup ];
    J = [J polytope(yJ)];
end
