% min  c'*Z
% s.t. AA*Z <= BB
%      GG*Z = HH
function [Vu] = norm1(VV,sysStruct,probStruct)
% number of states, number of inputs
nx = size(sysStruct.A,2);
nu = size(sysStruct.B,2);
N = probStruct.N;

% matrices for the objective function and the constraints for N = 1
H = [eye(nx);-eye(nx)];
L = [eye(nu);-eye(nu)];
K = [sysStruct.xmax; -sysStruct.xmin];
M = [sysStruct.umax ; -sysStruct.umin];

% matrices for the objective function and the constraints for N
tQ = kron(eye(N),probStruct.Q);
tR = kron(eye(N),probStruct.R);
tH = kron(eye(N),H);
tL = kron(eye(N),L);
tK = kron(ones(N,1),K);
tM = kron(ones(N,1),M);
tE1 = eye(nx);
tE2 = kron(ones(N-1,1),zeros(nx));
tE = [tE1;tE2];
tA1 = kron(ones(1,N),zeros(nx));
tA2 = kron(eye(N-1),sysStruct.A);
tA3 = kron(ones(N-1,1),zeros(nx));
tA = [tA1;tA2 tA3];
tB1 = kron(ones(1,N),zeros(nx,nu));
tB2 = kron(eye(N-1),sysStruct.B);
tB3 = kron(ones(N-1,1),zeros(nx,nu));
tB = [tB1;tB2 tB3];

% optimization variables
U = sdpvar(N*nu,1);
X = sdpvar(N*nx,1);
Eu = sdpvar(N*nu,1);
Ex = sdpvar(N*nx,1);

% objective and constraints
obj = 0; F = [];

ZnuH = zeros(size(tH,1),N*nu);
ZnxH = zeros(size(tH,1),N*nx);
ZnxL = zeros(size(tL,1),N*nx);
ZnuQ = zeros(size(tQ,1),N*nu);
ZnuL = zeros(size(tL,1),N*nu);
ZnxR = zeros(size(tR,1),N*nx);

AA = [ZnuH tH ZnuH ZnxH; ...
    tL ZnxL ZnuL ZnxL; ...
    ZnuQ tQ ZnuQ -eye(N*nx); ...
    ZnuQ -tQ ZnuQ -eye(N*nx); ...
    tR ZnxR -eye(N*nu) ZnxR; ...
    -tR ZnxR -eye(N*nu) ZnxR];

BB = [tK;tM;zeros(size(AA,1) - size([tK;tM],1),1)];
GG = [-tB (eye(N*nx)-tA) zeros(N*nx,N*nu) zeros(N*nx)];
% HH = tE*xt;
%
% F = [AA*[U;X;Eu;Ex] <= BB; ...
%     GG*[U;X;Eu;Ex] == HH];

% simulation
for i = 1:length(VV)
    u0opt = [];
    for j = 1:2
        HH = tE*VV{i}(j,:);
        
        F = [AA*[U;X;Eu;Ex] <= BB; ...
            GG*[U;X;Eu;Ex] == HH];
        
        info = solvesdp(F,obj);
        if info.problem ~= 0, error('Problem is Infeasible'),end
        u0opt = [u0opt double(U(1:nu))];
    end
    Vu{i} = u0opt';
end

