% min  X'*Q*X + U'*R*U
% s.t.    X == tA*X + tB*U +tE*x
%      tH*X <= tK
%      tL*U <= tM
function [Vu] = norm2(VV,sysStruct,probStruct)
% basic parameters
N = probStruct.N;             % prediction horizon
nx = size(sysStruct.A,2);     % number of states
nu = size(sysStruct.B,2);     % number of inputs

% matrices for the objective function and the constraints for N = 1
H = [eye(nx);-eye(nx)];
L = [eye(nu);-eye(nu)];
K = [sysStruct.xmax; -sysStruct.xmin];
M = [sysStruct.umax ; -sysStruct.umin];

% matrices for the objective function and the constraints for N
tQ = kron(eye(N),probStruct.Q);
tR = kron(eye(N),probStruct.R);
tH = kron(eye(N),H);
tL = kron(eye(N),L);
tK = kron(ones(N,1),K);
tM = kron(ones(N,1),M);
tE1 = eye(nx);
tE2 = kron(ones(N-1,1),zeros(nx));
tE = [tE1;tE2];
tA1 = kron(ones(1,N),zeros(nx));
tA2 = kron(eye(N-1),sysStruct.A);
tA3 = kron(ones(N-1,1),zeros(nx));
tA = [tA1;tA2 tA3];
tB1 = kron(ones(1,N),zeros(nx,nu));
tB2 = kron(eye(N-1),sysStruct.B);
tB3 = kron(ones(N-1,1),zeros(nx,nu));
tB = [tB1;tB2 tB3];

% optimization variables
U = sdpvar(N*nu,1);
X = sdpvar(N*nx,1);

% simulation
for i = 1:length(VV)
    u0opt = [];
    for j = 1:2
        % objective function and constraints
        obj = X'*tQ*X + U'*tR*U;
        F = [X == tA*X + tB*U +tE*VV{i}(j,:); ...
            tH*X <= tK;
            tL*U <= tM];
        
        info = solvesdp(F,obj);
        if info.problem ~= 0, error('Problem is infeasible'), end
        u0opt = [u0opt double(U(1:nu))];
    end
    Vu{i} = u0opt';
end
%%
% nx = size(sysStruct.A,2);
% nu = size(sysStruct.B,2);
% 
% % variable declarations
% for k = 1:probStruct.N + 1, x{k} = sdpvar(nx,1); end
% for k = 1:probStruct.N, u{k} = sdpvar(nu,1); end
% 
% obj = 0; F = [];
% for k = 1:probStruct.N
%     obj = obj + ...
%         x{k}'*probStruct.Q*x{k} + u{k}'*probStruct.R*u{k};
%     F = F + ...
%         [x{k+1} == sysStruct.A*x{k} + sysStruct.B*u{k}; ...
%         sysStruct.xmin <= x{k} <= sysStruct.xmax; ...
%         sysStruct.umin <= u{k} <= sysStruct.umax];
% end
% 
% % simulation
% for i = 1:length(VV)
%     u0opt = [];
%     for j = 1:2
%         info = solvesdp(F + [x{1} == VV{i}(:,j)], obj);
% %         if info.problem ~= 0, error('Problem is infeasible'), end
%         u0opt = [u0opt double(u{1})];
%     end
%     Vu{i} = u0opt;
% end