% min  sum(Ex) + sum(Eu)
% s.t.    X == tA*X+tB*U+tE*xt
%      tH*X <= tK
%      tL*U <= tM
%      -tEx <= tQ*X <= tEx
%      -tEu <= tR*U <= tEu
function [Vu] = norminf(VV,sysStruct,probStruct)
% number of states, number of inputs
nx = size(sysStruct.A,2);   % number of states
nu = size(sysStruct.B,2);   % number of inputs
N = probStruct.N;           % prediction horizon

% matrices for the objective function and the constraints for N = 1
H = [eye(nx);-eye(nx)];
L = [eye(nu);-eye(nu)];
K = [sysStruct.xmax; -sysStruct.xmin];
M = [sysStruct.umax ; -sysStruct.umin];

% matrices for the objective function and the constraints for N
tQ = kron(eye(N),probStruct.Q);
tR = kron(eye(N),probStruct.R);
tH = kron(eye(N),H);
tL = kron(eye(N),L);
tK = kron(ones(N,1),K);
tM = kron(ones(N,1),M);
tE1 = eye(nx);
tE2 = kron(ones(N-1,1),zeros(nx));
tE = [tE1;tE2];
tA1 = kron(ones(1,N),zeros(nx));
tA2 = kron(eye(N-1),sysStruct.A);
tA3 = kron(ones(N-1,1),zeros(nx));
tA = [tA1;tA2 tA3];
tB1 = kron(ones(1,N),zeros(nx,nu));
tB2 = kron(eye(N-1),sysStruct.B);
tB3 = kron(ones(N-1,1),zeros(nx,nu));
tB = [tB1;tB2 tB3];

% optimization variables
U = sdpvar(N*nu,1);
X = sdpvar(N*nx,1);
Eu = sdpvar(N,1);
Ex = sdpvar(N,1);
tEx = kron(Ex, ones(nx,1));
tEu = kron(Eu, ones(nu,1));

% simulation
for i = 1:length(VV)
    u0opt = [];
    for j = 1:2
        % objective function and constraints
        obj = sum(Ex) + sum(Eu);
        F = [X == tA*X+tB*U+tE*VV{i}(j,:); ...
            tH*X <= tK; ...
            tL*U <= tM; ...
            -tEx <= tQ*X <= tEx; ...
            -tEu <= tR*U <= tEu];
        info = solvesdp(F,obj);
        if info.problem ~= 0, error('Problem is infeasible'), end
        u0opt = [u0opt double(U(1:nu))];
    end
    Vu{i} = u0opt';
end